/*
	Crossing
	Copyright (C) 2007 Michal Vaner (michal.vaner@kdemail.net)

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	he Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the

	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include "line.h"
#include "point.h"
#include "globals.h"
#include "load.h"
#include "vehicle.h"

#include <QDomElement>
#include <QtGlobal>
#include <QCoreApplication>
#include <math.h>

Line::~Line() {
}

#define EXCP_IP XMLLoadDataE(QCoreApplication::translate("xml_errors", "Invalid or missing point ID in line"))

Line::Line(const QDomElement &node, const QHash<QString, Point *> &points, Indexer &indexer) :
	Indexable(Indexable::Structural, qMax(notNull(points.value(node.attribute("from")), EXCP_IP)->index(), notNull(points.value(node.attribute("to")), EXCP_IP)->index())),
	mId(nonEmpty(node.attribute("id"), XMLLoadDataE(QCoreApplication::translate("xml_errors", "Invalid line ID"))))
{
	//Link into the structure
	indexer << this;
	mDest = points.value(node.attribute("to"));//They can't be NULL now, they were checked in the entrance
	mSrc = points.value(node.attribute("from"));
	int xDist = mSrc->x() - mDest->x(), yDist = mSrc->y() - mDest->y();
	double dist = sqrt(xDist * xDist + yDist * yDist);
	mXInc = xDist / dist;
	mYInc = yDist / dist;
	if(xDist)
		mAngle = int(180 * atan(double(yDist) / double(xDist)) / M_PI);
	else
		if(yDist < 0)
			mAngle = 270;
		else
			mAngle = 90;
	if(xDist < 0)
		mAngle += 180;
	mLength = dist;
	mSrc->pushLine(this);
	mDest->pushBackLine(this);
}

void Line::enter(::Vehicle *vehicle) {
	if(ongoing.count())
		vehicle->waitForVehicle = ongoing.last();
	ongoing.append(vehicle);
}

void Line::leave() {
	ongoing.removeFirst();
	if(ongoing.count())
		ongoing.first()->waitForVehicle = NULL;
}

void Line::clearOngoings() {
	ongoing.clear();
}
